@@ -307,10 +307,23 @@ getmntdevbackward (const char *name, struct mntentchn *mcprev) {
 	mc0 = mtab_head();
 	if (!mcprev)
 		mcprev = mc0;
+
+	/* canonical names in mtab */
 	for (mc = mcprev->prev; mc && mc != mc0; mc = mc->prev) {
 		if (streq(mc->m.mnt_fsname, name))
 			return mc;
 	}
+
+	/* non-canonical names in mtab (this is BAD THING!) */
+	for (mc = mcprev->prev; mc && mc != mc0; mc = mc->prev) {
+		char *cn = canonicalize(mc->m.mnt_fsname);
+		int res = cn ? streq(cn, name) : 0;
+
+		free(cn);
+		if (res)
+			return mc;
+	}
+
 	return NULL;
 }
 
@@ -442,7 +455,13 @@ getfs_by_dir (const char *dir) {
 
 	cdir = canonicalize(dir);
 	for (mc = mc0->nxt; mc && mc != mc0; mc = mc->nxt) {
-		if (streq(mc->m.mnt_dir, cdir)) {
+		int ok = streq(mc->m.mnt_dir, cdir);
+		if (!ok) {
+			char *dr = canonicalize(mc->m.mnt_dir);
+			ok = dr ? streq(dr, cdir) : 0;
+			free(dr);
+		}
+		if (ok) {
 			free(cdir);
 			return mc;
 		}
@@ -561,25 +580,10 @@ handler (int sig) {
 }
 
 static void
-setlkw_timeout (int sig) {
+setlkw_timeout (int sig __attribute__ ((__unused__))) {
      /* nothing, fcntl will fail anyway */
 }
 
-#ifdef HAVE_LIBMOUNT_MOUNT
-static struct libmnt_lock *libmount_lock;
-
-struct libmnt_lock *
-init_libmount_lock(const char *filename)
-{
-	if (filename)
-		return libmount_lock = mnt_new_lock(filename, 0);
-	if (libmount_lock)
-		mnt_free_lock(libmount_lock);
-	libmount_lock = NULL;
-	return NULL;
-}
-#endif
-
 /* Remove lock file.  */
 void
 unlock_mtab (void) {
@@ -589,10 +593,6 @@ unlock_mtab (void) {
 		unlink (_PATH_MOUNTED_LOCK);
 		we_created_lockfile = 0;
 	}
-#ifdef HAVE_LIBMOUNT_MOUNT
-	if (libmount_lock)
-		mnt_unlock_file(libmount_lock);
-#endif
 }
 
 /* Create the lock file.
@@ -775,7 +775,7 @@ get_option(const char *optname, const char *src, size_t *len)
 		return NULL;
 
 	end = strchr(opt, ',');
-	sz = end ? end - opt : strlen(opt);
+	sz = end && end > opt ? (size_t) (end - opt) : strlen(opt);
 
 	if (len)
 		*len = sz;
@@ -877,7 +877,7 @@ update_mtab (const char *dir, struct my_mntent *instead) {
 	mntFILE *mfp, *mftmp;
 	const char *fnam = _PATH_MOUNTED;
 	struct mntentchn mtabhead;	/* dummy */
-	struct mntentchn *mc, *mc0, *absent = NULL;
+	struct mntentchn *mc, *mc0 = NULL, *absent = NULL;
 	struct stat sbuf;
 	int fd;
 
@@ -901,10 +901,12 @@ update_mtab (const char *dir, struct my_mntent *instead) {
 	read_mntentchn(mfp, fnam, mc);
 
 	/* find last occurrence of dir */
-	for (mc = mc0->prev; mc && mc != mc0; mc = mc->prev)
-		if (streq(mc->m.mnt_dir, dir))
-			break;
-	if (mc && mc != mc0) {
+	if (dir) {
+		for (mc = mc0->prev; mc && mc != mc0; mc = mc->prev)
+			if (streq(mc->m.mnt_dir, dir))
+				break;
+	}
+	if (dir && mc && mc != mc0) {
 		if (instead == NULL) {
 			/* An umount - remove entry */
 			if (mc && mc != mc0) {
@@ -950,24 +952,37 @@ update_mtab (const char *dir, struct my_mntent *instead) {
 		int errsv = errno;
 		error (_("cannot open %s (%s) - mtab not updated"),
 		       _PATH_MOUNTED_TMP, strerror (errsv));
-		discard_mntentchn(mc0);
 		goto leave;
 	}
 
 	for (mc = mc0->nxt; mc && mc != mc0; mc = mc->nxt) {
 		if (my_addmntent(mftmp, &(mc->m)) == 1) {
 			int errsv = errno;
-			die (EX_FILEIO, _("error writing %s: %s"),
+			error(_("error writing %s: %s"),
 			     _PATH_MOUNTED_TMP, strerror (errsv));
+			goto leave;
 		}
 	}
 
 	discard_mntentchn(mc0);
+	mc0 = NULL;
+
+	/*
+	 * We have to be paranoid with write() to avoid incomplete
+	 * /etc/mtab. Users are able to control writing by RLIMIT_FSIZE.
+	 */
+	if (fflush(mftmp->mntent_fp) != 0) {
+		int errsv = errno;
+		error (_("%s: cannot fflush changes: %s"),
+				_PATH_MOUNTED_TMP, strerror (errsv));
+		goto leave;
+	}
+
 	fd = fileno(mftmp->mntent_fp);
 
 	/*
-	 * It seems that better is incomplete and broken /mnt/mtab that
-	 * /mnt/mtab that is writeable for non-root users.
+	 * It seems that better is incomplete and broken /etc/mtab that
+	 * /etc/mtab that is writeable for non-root users.
 	 *
 	 * We always skip rename() when chown() and chmod() failed.
 	 * -- kzak, 11-Oct-2007
@@ -1004,6 +1019,9 @@ update_mtab (const char *dir, struct my_mntent *instead) {
 	}
 
  leave:
+	if (mc0)
+		discard_mntentchn(mc0);
+	unlink(_PATH_MOUNTED_TMP);
 	unlock_mtab();
 }
 
